@@ -3,7 +3,7 @@ names(items) <- paste0("i",1:30)
 persons <- 500
 mean <- 0
 sd <- 1
-dat <- tmt:::sim.rm(persons, items,1111)
+dat <- tmt:::sim.rm(persons, items,c(1111,1112))
 invisible(dat.rm <- tmt_rm(dat, optimization = "optim"))
 suppressWarnings(dat.lrt <- tmt_lrtest(dat.rm, 
 	split = "median", optimization = "optim"))
@@ -31,7 +31,7 @@ context("test-tmt_gmc")
 # -----------------------------------------------------------------
   test_that("tmt_gmc data structure", {
     expect_type(p,"list")
-    expect_that(length(p), equals(9))
+    expect_that(length(p), equals(11))
   })
   test_that("tmt_gmc class", {
        expect_is(
